# Copyrights 2012 by [Mark Overmeer].
#  For other contributors see ChangeLog.
# See the manual pages for details on the licensing terms.
# Pod stripped from pm file by OODoc 2.00.
use warnings;
use strict;

package Mozilla::Persona;
use vars '$VERSION';
$VERSION = '0.12';

use open 'utf8';

use Log::Report    qw/persona/;


1;
